// 
// Copyright (C) 2015, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.Gui.Tools;
using NinjaTrader.NinjaScript.DrawingTools;
using SharpDX;
using SharpDX.Direct2D1;
using Brush = System.Windows.Media.Brush;
using Ellipse = SharpDX.Direct2D1.Ellipse;

#endregion

public enum SqueezeMaTypeEnum
{
    EMA,
    DEMA,
    TEMA
}
// This namespace holds indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.Sim22.Gapless
{
    /// <summary>
    /// The Sim22_SqueezeGapless is 'cloned' from the well known TTM squeeze. 
	/// It is dubious as to whether this is actually copyrighted since it was originally posted as 'Nick's Next Move' in the Tradestation forums many years ago - absolutely free.
	/// 
	/// * I have scrapped the 'multiplier' input from the original squeeze, and introduced a 'Tick size' option that automatically adjusts to the tick size of the instrument. 
	/// This makes it easier to compare instruments of different tick sizes.
	/// * The user can also set the histogram to automatically adjust its width with the chart bar size.
    /// 
	/// Gapless:
	/// It includes a gapless option, which when used intraday, the session opening gap is ignored. When using >= Daily bars all open gaps are ignored. 
	/// This is in a bid to help intraday traders have meaningful momentum direction just from the prices that are actually trading, not from tradeless gaps.
    /// Original gapless concept sourced from the TradeStation forums.
	/// 
    /// Sim22 Oct 2016 NT8b13.
    /// </summary>
    public class Sim22_SqueezeGapless : Indicator
    {
      
        private StdDev stdDev;
        private LinReg linReg;
        private EMA ema;
        private DEMA dema;
        private TEMA tema;
        private MAX max;
        private MIN min;

        private Series<double> maSeries;
        private Series<bool> squeezeAlertBoolSeries;
        private Series<double> linValueSeries;
        private Series<double> atrSeries;
        private Series<double> highSeries;
        private Series<double> lowSeries;
        private Series<double> closeSeries;
        private Series<int> numBarsSeries;

        private double gap;
        private const double AlertLevel = 1.0;
        private double tickSize;
        private double squeezeVar;
        private int signalBarNum;
        
        private Brush tempBrush = null;
        private double previousInput = 0d;
        protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"TTM Squeeze gapless. Sim22 Oct 2016 NT8b12";
				Name						= "Sim22_SqueezeGapless";
				
				Calculate 					= Calculate.OnPriceChange;
				IsSuspendedWhileInactive	= true;				
				IsOverlay					= false;

			    SqueezeLength               = 20;
			    MaLength                    = 20;
			    LinRegLength                = 20;
			    StdDevBollinger             = 2.0;
			    StdDevKeltner               = 1.5;
			    UseGapless                  = false;
			    UseTickSize                 = true;
			    UsePriceBarWidth            = true;
			    BarOpacity                  = 70;
			    PaintBars                   = true;
                HistabvZeroDwnColor         = Brushes.DarkBlue;
                HistabvZeroUpColor          = Brushes.Blue;
                HistblwZeroDwnColor         = Brushes.Red;
                HistblwZeroUpColor          = Brushes.DarkRed;
                AlertNormalColor            = Brushes.Red;
			    AlertSignalColor            = Brushes.Lime;

                SqueezeMaType               = SqueezeMaTypeEnum.EMA;

                AddPlot(new Stroke(Brushes.Gray, 6), PlotStyle.Bar, "Histogram");
            }
			else if (State == State.Configure)
			{
			    HistabvZeroDwnColor.Freeze();
                HistabvZeroUpColor.Freeze();
                HistblwZeroDwnColor.Freeze();
                HistblwZeroUpColor.Freeze();
                AlertNormalColor.Freeze();
                AlertSignalColor.Freeze();

                highSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                lowSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                closeSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                
                linValueSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                squeezeAlertBoolSeries = new Series<bool>(this, MaximumBarsLookBack.Infinite);
                atrSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                numBarsSeries = new Series<int>(this, MaximumBarsLookBack.Infinite);
                maSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);

                stdDev  = StdDev(closeSeries, SqueezeLength);              
                linReg  = LinReg(linValueSeries, LinRegLength);
                max     = MAX(highSeries, MaLength);
                min     = MIN(lowSeries, MaLength);

                tickSize = UseTickSize ? TickSize : 1;

			    switch (SqueezeMaType)
			    {
			        case SqueezeMaTypeEnum.EMA:
                        ema = EMA(closeSeries, MaLength);
                        break;
			        case SqueezeMaTypeEnum.DEMA:
                        dema = DEMA(closeSeries, MaLength);
                        break;
			        case SqueezeMaTypeEnum.TEMA:
                        tema = TEMA(closeSeries, MaLength);
                        break;
			    }
			}
			else if (State == State.Historical)
			{
			}
		}

        public override string DisplayName
        {
            get { return "Sim22 Squeeze " + (UseGapless ? " 'Gapless' " : "") + (UseTickSize ? " 'TickSize' " : "") + "(Sq: " + SqueezeLength + "/" + StdDevBollinger + "/" + StdDevKeltner + "/Hi: " + SqueezeMaType + "/" + MaLength + "/" + LinRegLength + ")" ; }
        }

        public override void OnCalculateMinMax()
        {
            try
            {
                if (Bars == null)
                    return;

                int lastBar = ChartBars.ToIndex;
                int firstBar = ChartBars.FromIndex;

                double minValue = double.MaxValue;
                double maxValue = double.MinValue;

                for (int index = firstBar; index <= lastBar; index++)
                {
                    {
                        minValue = Math.Min(minValue, Values[0].GetValueAt(index));
                        maxValue = Math.Max(maxValue, Values[0].GetValueAt(index));
                    }
                }

                if (minValue > 0)
                    minValue = 0;
                if (maxValue < 0)
                    maxValue = 0;
                if ((maxValue - minValue) < 1)
                {
                    minValue -= 1;
                    maxValue += 1;
                }

                MinValue = minValue;
                MaxValue = maxValue;
            }
            catch (Exception ex)
            {
                Print("Value Chart OnCalculateMinMax: " + ex);
            }
        }

        // Need a custom ATR method to take into account gapless high and low values
        protected double CalculateAtr(int period, double high, double low, double close1, ISeries<double> iAtrSeries)
        {
            if (CurrentBar == 0)
            {
                return (high - low);
            }

            double trueRange = Math.Max(Math.Abs(low - close1), Math.Max(high - low, Math.Abs(high - close1)));
            return ((Math.Min(CurrentBar + 1, period) - 1)*iAtrSeries[1] + trueRange)/Math.Min(CurrentBar + 1, period);
        }

        protected override void OnBarUpdate()
        {
            try
            {
                // Do not allow plotting with non-price data
                if (!(Input is PriceSeries))
                    return;

                if (CurrentBar < 0)
                    return;

                // For tick replay mode
                if (IsTickReplays[0] == true)
                {
                    double currentInput = Input[0];
                    //calculate every price change instead of every tick
                    if (!IsFirstTickOfBar && previousInput.ApproxCompare(currentInput) == 0)
                        return;
                    previousInput = currentInput;
                }
                if (CurrentBar == 0)
                {
                    gap = 0d;
                }

                if (UseGapless)
                {
                    // If EOD or (if intraday and first bar of session)
                    if (IsFirstTickOfBar && CurrentBar > 0 && (!Bars.BarsType.IsIntraday || (Bars.IsFirstBarOfSession && Bars.IsResetOnNewTradingDay)))
                    {
                        gap += (Open[0] - Close[1]);
                    }
                }
                // If not gapless, gap = 0 so that's okay.
                closeSeries[0] = Close[0] - gap;
                highSeries[0] = High[0] - gap;
                lowSeries[0] = Low[0] - gap;

                if (CurrentBar == 0)
                    return;

                double high = highSeries[0];
                double low = lowSeries[0];
                double close1 = closeSeries[1];

                atrSeries[0] = CalculateAtr(SqueezeLength, high, low, close1, atrSeries);

                double stdDev0 = stdDev[0];
                double var0 = StdDevKeltner*atrSeries[0];

                if (var0.ApproxCompare(0) != 0)
                {
                    squeezeVar = StdDevBollinger*stdDev0/var0;
                }
                if (squeezeVar < AlertLevel)
                {
                    // Squeeze
                    squeezeAlertBoolSeries[0] = false;
                    if (!squeezeAlertBoolSeries[1])
                        signalBarNum = CurrentBar;
                }
                else
                {
                    squeezeAlertBoolSeries[0] = true;
                    if (squeezeAlertBoolSeries[1])
                        signalBarNum = CurrentBar;
                }
                // Set bars ago. Include current bar
                numBarsSeries[0] = CurrentBar - signalBarNum + 1;

                switch (SqueezeMaType)
                {
                    case SqueezeMaTypeEnum.EMA:
                        maSeries[0] = ema[0];
                        break;
                    case SqueezeMaTypeEnum.DEMA:
                        maSeries[0] = dema[0];
                        break;
                    case SqueezeMaTypeEnum.TEMA:
                        maSeries[0] = tema[0];
                        break;
                }

                var linValue0 = closeSeries[0] - ((max[0] + min[0])/2.0 + maSeries[0])*0.5;

                linValueSeries[0] = linValue0;

                var histogramValue0 = linReg[0]/tickSize;

                Value[0] = histogramValue0;

                var histogramValue1 = Value[1];

                if (histogramValue0 >= 0.0)
                {
                    if (histogramValue0 > histogramValue1)
                    {
                        PlotBrushes[0][0] = HistabvZeroUpColor;
                        if (PaintBars)
                        {
                            tempBrush = HistabvZeroUpColor.Clone();
                            tempBrush.Opacity = BarOpacity*0.01f;
                            BarBrushes[0] = tempBrush;

                            CandleOutlineBrushes[0] = HistabvZeroUpColor;
                        }
                    }
                    else
                    {
                        PlotBrushes[0][0] = HistabvZeroDwnColor;
                        if (PaintBars)
                        {
                            tempBrush = HistabvZeroUpColor.Clone();
                            tempBrush.Opacity = BarOpacity*0.01f;
                            BarBrushes[0] = HistabvZeroDwnColor;

                            CandleOutlineBrushes[0] = HistabvZeroDwnColor;
                        }
                    }
                }
                else
                {
                    if (histogramValue0 < histogramValue1)
                    {
                        PlotBrushes[0][0] = HistblwZeroDwnColor;
                        if (PaintBars)
                        {
                            tempBrush = HistabvZeroUpColor.Clone();
                            tempBrush.Opacity = BarOpacity*0.01f;
                            BarBrushes[0] = HistblwZeroDwnColor;

                            CandleOutlineBrushes[0] = HistblwZeroDwnColor;
                        }
                    }
                    else
                    {
                        PlotBrushes[0][0] = HistblwZeroUpColor;
                        if (PaintBars)
                        {
                            tempBrush = HistabvZeroUpColor.Clone();
                            tempBrush.Opacity = BarOpacity*0.01f;
                            BarBrushes[0] = HistblwZeroUpColor;

                            CandleOutlineBrushes[0] = HistblwZeroUpColor;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Print(Name + ": " + ex);
            }
        }

        protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
        {
            if (Bars == null || ChartControl == null || Bars.Instrument == null || !IsVisible)
            {
                return;
            }

            try
            {
                // Temporarily initialize the brushes
                SharpDX.Direct2D1.Brush tempBarbrushDx = Brushes.Gray.ToDxBrush(RenderTarget);
                SharpDX.Direct2D1.Brush tempBarOutlinebrushDx = Brushes.DimGray.ToDxBrush(RenderTarget);
                SharpDX.Direct2D1.Brush tempAlertBrushDx = Brushes.Black.ToDxBrush(RenderTarget);

                Ellipse ellipse = new Ellipse();
                RectangleF histoRectF = new RectangleF();
                RoundedRectangle histoRoundedRect = new RoundedRectangle();
                float lineWidth = Plots[0].Width;

                if (UsePriceBarWidth)
                {
                    lineWidth = (float) chartControl.GetBarPaintWidth(ChartBars);

                    if (ChartBars.Properties.ChartStyleType == ChartStyleType.OHLC)
                    {
                        float barWidth = (float) Math.Max(1, chartControl.BarWidth);

                        lineWidth = (float) Math.Round(barWidth, 0);
                    }
                }

                lineWidth = Math.Min(chartControl.Properties.BarDistance - 1, lineWidth); // limit bar width

                if (float.IsNaN(lineWidth)) //just in case there are no values
                {
                    lineWidth = 6f;
                }

                float ellipseRadius = Math.Min((chartControl.Properties.BarDistance - 1)*0.5f, lineWidth*0.7f); // limit dot width

                //if (lineWidth >= chartControl.Properties.BarDistance - 1)
                //    ellipseRadius = lineWidth * 0.5f;

                for (int idx = ChartBars.FromIndex; idx <= ChartBars.ToIndex; idx++)
                {
                    // Do not plot the last values if set to OnBarClose
                    if (Calculate == Calculate.OnBarClose && idx == ChartBars.ToIndex)
                        continue;

                    double histValue = Values[0].GetValueAt(idx);
                    //in Pixels
                    float histY = chartScale.GetYByValue(histValue);
                    float zeroY = chartScale.GetYByValue(0);
                    int x = chartControl.GetXByBarIndex(ChartBars, idx);

                    // Bars
                    histoRectF.Height = histY - zeroY;
                    histoRectF.X = x - lineWidth*0.5f;
                    histoRectF.Y = zeroY;
                    histoRectF.Width = lineWidth;
                    histoRoundedRect.Rect = histoRectF;
                    histoRoundedRect.RadiusX = lineWidth*0.4f;
                    histoRoundedRect.RadiusY = lineWidth*0.4f;

                    if (PlotBrushes[0].Get(idx) != null)
                    {
                        tempBarbrushDx = PlotBrushes[0].Get(idx).ToDxBrush(RenderTarget);
                        tempBarOutlinebrushDx = PlotBrushes[0].Get(idx).ToDxBrush(RenderTarget);
                    }

                    tempBarbrushDx.Opacity = BarOpacity*0.01f;
                    tempBarOutlinebrushDx.Opacity = 1.0f;
                    // Round edges nicely
                    RenderTarget.AntialiasMode = SharpDX.Direct2D1.AntialiasMode.PerPrimitive;
                    // Draw Bars
                    RenderTarget.FillRoundedRectangle(histoRoundedRect, tempBarbrushDx);
                    RenderTarget.DrawRoundedRectangle(histoRoundedRect, tempBarOutlinebrushDx, 1f);
                    // Dot
                    Vector2 vector2 = new Vector2(x, zeroY);
                    ellipse.Point = vector2;
                    ellipse.RadiusY = ellipse.RadiusX = ellipseRadius;

                    if (squeezeAlertBoolSeries.IsValidDataPointAt(idx))
                    {
                        tempAlertBrushDx = squeezeAlertBoolSeries.GetValueAt(idx) ? AlertSignalColor.ToDxBrush(RenderTarget) : AlertNormalColor.ToDxBrush(RenderTarget);
                    }

                    RenderTarget.FillEllipse(ellipse, tempAlertBrushDx);
                    RenderTarget.DrawEllipse(ellipse, Brushes.Black.ToDxBrush(RenderTarget), 1f);
                    RenderTarget.AntialiasMode = SharpDX.Direct2D1.AntialiasMode.Aliased;
                }

                tempBarbrushDx.Dispose();
                tempAlertBrushDx.Dispose();
                tempBarOutlinebrushDx.Dispose();
            }
            catch (Exception ex)
            {
                Print(Name + ": " + ex);
            }
        }

        #region Properties

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> HistogramSeries
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<bool> SqueezeAlertBoolSeries
        {
            get { return squeezeAlertBoolSeries; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<int> NumberOfBarsSeries
        {
            get { return numBarsSeries; }
        }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Plot gapless?", GroupName = "01 Parameters.General", Order = 1)]
        public bool UseGapless { get; set; }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Use tick size?", Description = "Divide by 'tick size' to compare with other instruments", GroupName = "01 Parameters.General", Order = 2)]
        public bool UseTickSize { get; set; }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "MA type", GroupName = "02 Parameters.Histogram", Order = 1)]
        public SqueezeMaTypeEnum SqueezeMaType { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "MA length", GroupName = "02 Parameters.Histogram", Order = 2)]
        public int MaLength { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Linear Reg length", GroupName = "02 Parameters.Histogram", Order = 3)]
        public int LinRegLength { get; set; }

        [Range(1, double.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "BB value", GroupName = "03 Parameters.Squeeze", Order = 1)]
        public double StdDevBollinger { get; set; }

        [Range(1, double.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "K value", GroupName = "03 Parameters.Squeeze", Order = 2)]
        public double StdDevKeltner { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Length", GroupName = "03 Parameters.Squeeze", Order = 3)]
        public int SqueezeLength { get; set; }
      
        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Use price bar width?", Description = "Auto adjust the histogram to the width of the chart bars.", GroupName = "04 Plots", Order = 1)]
        public bool UsePriceBarWidth { get; set; }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Paint bars & Outline?", Description = "Paint bars the histogram colors.", GroupName = "04 Plots", Order = 2)]
        public bool PaintBars { get; set; }

        [Range(0, 100), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Bar opacity % (0-100)", GroupName = "05 Histogram Colors", Order = 1)]
        public int BarOpacity { get; set; }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Above up color", Order = 2, GroupName = "05 Histogram Colors")]
        public Brush HistabvZeroUpColor { get; set; }

        [Browsable(false)]
        public string HistabvZeroUpColorSerialize
        {
            get { return Serialize.BrushToString(HistabvZeroUpColor); }
            set { HistabvZeroUpColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Above down color", Order = 3, GroupName = "05 Histogram Colors")]
        public Brush HistabvZeroDwnColor { get; set; }

        [Browsable(false)]
        public string HistabvZeroDwnColorSerialize
        {
            get { return Serialize.BrushToString(HistabvZeroDwnColor); }
            set { HistabvZeroDwnColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Below up color", Order = 4, GroupName = "05 Histogram Colors")]
        public Brush HistblwZeroUpColor { get; set; }

        [Browsable(false)]
        public string HistblwZeroUpColorSerialize
        {
            get { return Serialize.BrushToString(HistblwZeroUpColor); }
            set { HistblwZeroUpColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Below down color", Order = 5, GroupName = "05 Histogram Colors")]
        public Brush HistblwZeroDwnColor { get; set; }

        [Browsable(false)]
        public string HistblwZeroDwnColorSerialize
        {
            get { return Serialize.BrushToString(HistblwZeroDwnColor); }
            set { HistblwZeroDwnColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Squeeze color", Order = 6, GroupName = "06 Squeeze Colors")]
        public Brush AlertNormalColor { get; set; }

        [Browsable(false)]
        public string AlertNormalColorSerialize
        {
            get { return Serialize.BrushToString(AlertNormalColor); }
            set { AlertNormalColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Normal color", Order = 7, GroupName = "06 Squeeze Colors")]
        public Brush AlertSignalColor { get; set; }

        [Browsable(false)]
        public string AlertSignalColorSerialize
        {
            get { return Serialize.BrushToString(AlertSignalColor); }
            set { AlertSignalColor = Serialize.StringToBrush(value); }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Sim22.Gapless.Sim22_SqueezeGapless[] cacheSim22_SqueezeGapless;
		public Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			return Sim22_SqueezeGapless(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength, usePriceBarWidth, paintBars, barOpacity);
		}

		public Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(ISeries<double> input, bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			if (cacheSim22_SqueezeGapless != null)
				for (int idx = 0; idx < cacheSim22_SqueezeGapless.Length; idx++)
					if (cacheSim22_SqueezeGapless[idx] != null && cacheSim22_SqueezeGapless[idx].UseGapless == useGapless && cacheSim22_SqueezeGapless[idx].UseTickSize == useTickSize && cacheSim22_SqueezeGapless[idx].SqueezeMaType == squeezeMaType && cacheSim22_SqueezeGapless[idx].MaLength == maLength && cacheSim22_SqueezeGapless[idx].LinRegLength == linRegLength && cacheSim22_SqueezeGapless[idx].StdDevBollinger == stdDevBollinger && cacheSim22_SqueezeGapless[idx].StdDevKeltner == stdDevKeltner && cacheSim22_SqueezeGapless[idx].SqueezeLength == squeezeLength && cacheSim22_SqueezeGapless[idx].UsePriceBarWidth == usePriceBarWidth && cacheSim22_SqueezeGapless[idx].PaintBars == paintBars && cacheSim22_SqueezeGapless[idx].BarOpacity == barOpacity && cacheSim22_SqueezeGapless[idx].EqualsInput(input))
						return cacheSim22_SqueezeGapless[idx];
			return CacheIndicator<Sim22.Gapless.Sim22_SqueezeGapless>(new Sim22.Gapless.Sim22_SqueezeGapless(){ UseGapless = useGapless, UseTickSize = useTickSize, SqueezeMaType = squeezeMaType, MaLength = maLength, LinRegLength = linRegLength, StdDevBollinger = stdDevBollinger, StdDevKeltner = stdDevKeltner, SqueezeLength = squeezeLength, UsePriceBarWidth = usePriceBarWidth, PaintBars = paintBars, BarOpacity = barOpacity }, input, ref cacheSim22_SqueezeGapless);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			return indicator.Sim22_SqueezeGapless(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength, usePriceBarWidth, paintBars, barOpacity);
		}

		public Indicators.Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(ISeries<double> input , bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			return indicator.Sim22_SqueezeGapless(input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength, usePriceBarWidth, paintBars, barOpacity);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			return indicator.Sim22_SqueezeGapless(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength, usePriceBarWidth, paintBars, barOpacity);
		}

		public Indicators.Sim22.Gapless.Sim22_SqueezeGapless Sim22_SqueezeGapless(ISeries<double> input , bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength, bool usePriceBarWidth, bool paintBars, int barOpacity)
		{
			return indicator.Sim22_SqueezeGapless(input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength, usePriceBarWidth, paintBars, barOpacity);
		}
	}
}

#endregion
