// 
// Copyright (C) 2015, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using SharpDX;
using SharpDX.Direct2D1;
using Brush = System.Windows.Media.Brush;
using Ellipse = SharpDX.Direct2D1.Ellipse;

#endregion

public enum SqueezeMaTypeEnumA
{
    EMA,
    DEMA,
    TEMA
}
// This namespace holds indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.Sim22.Gapless
{
    /// <summary>
	/// Analyzer version only of the squeeze!!
	/// 
    /// The Sim22_SqueezeGaplessA is 'cloned' from the well known TTM squeeze. 
	/// It is dubious as to whether this is actually copyrighted since it was originally posted as 'Nick's Next Move' in the Tradestation forums many years ago - absolutely free.
	/// 
	/// * I have scrapped the 'multiplier' input from the original squeeze, and introduced a 'Tick size' option that automatically adjusts to the tick size of the instrument. 
	/// This makes it easier to compare instruments of different tick sizes.
	/// * The user can also set the histogram to automatically adjust its width with the chart bar size.
    /// 
	/// Gapless:
	/// It includes a gapless option, which when used intraday, the session opening gap is ignored. When using >= Daily bars all open gaps are ignored. 
	/// This is in a bid to help intraday traders have meaningful momentum direction just from the prices that are actually trading, not from tradeless gaps.
    /// Original gapless concept sourced from the TradeStation forums.
	/// 
    /// Sim22 Oct 2016 NT8b13.
    /// </summary>
    public class Sim22_SqueezeGaplessA : Indicator
    {
      
        private StdDev stdDev;
        private LinReg linReg;
        private EMA ema;
        private DEMA dema;
        private TEMA tema;
        private MAX max;
        private MIN min;

        private Series<double> maSeries;
        private Series<double> linValueSeries;
        private Series<double> atrSeries;
        private Series<double> highSeries;
        private Series<double> lowSeries;
        private Series<double> closeSeries;
       
        private double gap;
        private const double AlertLevel = 1.0;
        private double tickSize;
        private double squeezeVar;
        private int signalBarNum;
       
        protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"TTM Squeeze gapless. Sim22 Oct 2016 NT8b12";
				Name						= "Sim22_SqueezeGaplessA";
				
				Calculate 					= Calculate.OnPriceChange;
				IsSuspendedWhileInactive	= true;				
				IsOverlay					= false;
                //Length                      = 20;
                SqueezeLength               = 20;
                MaLength                    = 20;
                LinRegLength                = 20;
                StdDevBollinger             = 2.0;
                StdDevKeltner               = 1.5;
                StdDevBollinger             = 2.0;
			    StdDevKeltner               = 1.5;
			    UseGapless                  = false;
			    UseTickSize                 = true;

                SqueezeMaType = SqueezeMaTypeEnum.EMA;

                AddPlot(new Stroke(Brushes.Gray, 1), PlotStyle.Line, "HistogramValue");
                AddPlot(new Stroke(Brushes.Gray, 1), PlotStyle.Line, "HistogramDirection");
                AddPlot(new Stroke(Brushes.Gray, 1), PlotStyle.Line, "Squeeze");
                AddPlot(new Stroke(Brushes.Gray, 1), PlotStyle.Line, "NumberOfBars");
            }
			else if (State == State.Configure)
            {
                highSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                lowSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                closeSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                
                linValueSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                atrSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
                maSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);

                stdDev = StdDev(closeSeries, SqueezeLength);
                linReg = LinReg(linValueSeries, LinRegLength);
                max = MAX(highSeries, MaLength);
                min = MIN(lowSeries, MaLength);

                tickSize = UseTickSize ? TickSize : 1;

                switch (SqueezeMaType)
                {
                    case SqueezeMaTypeEnum.EMA:
                        ema = EMA(closeSeries, MaLength);
                        break;
                    case SqueezeMaTypeEnum.DEMA:
                        dema = DEMA(closeSeries, MaLength);
                        break;
                    case SqueezeMaTypeEnum.TEMA:
                        tema = TEMA(closeSeries, MaLength);
                        break;
                }
            }
            else if (State == State.Historical)
            {
                
            }
        }

        public override string DisplayName
        {    
            get { return "Sim22 Squeeze " + (UseGapless ? " 'Gapless' " : "") + (UseTickSize ? " 'TickSize' " : "") + "(Sq: " + SqueezeLength + "/" + StdDevBollinger + "/" + StdDevKeltner + "/Hi: " + SqueezeMaType + "/" + MaLength + "/" + LinRegLength + ")"; }       
        }

        public override string FormatPriceMarker(double price)
        {
            return price.ToString("N0");
        }
        // Need a custom ATR method to take into account gapless high and low values
        protected double CalculateAtr(int period, double high, double low, double close1, ISeries<double> iAtrSeries)
        {          
            if (CurrentBar == 0)
            {               
                return (high - low);
            }

            double trueRange = Math.Max(Math.Abs(low - close1), Math.Max(high - low, Math.Abs(high - close1)));
            return ((Math.Min(CurrentBar + 1, period) - 1) * iAtrSeries[1] + trueRange) / Math.Min(CurrentBar + 1, period);
        }
       
        protected override void OnBarUpdate()
		{
		    try
		    {
                // If not the analyzer, return.
                if (!(Parent is MarketAnalyzerColumnBase))
                {
                    Draw.TextFixed(this, "SqueezeError", "You can only use this version of the squeeze in the market analyzer.", TextPosition.Center);
                    return;
                }

                // Do not allow plotting with non-price data
                if (!(Input is PriceSeries))
		            return;
		        if (CurrentBar < 0)
		            return;

                if (CurrentBar == 0)
		        {
		            gap = 0d;
		        }

                if (UseGapless)
		        {
                    // If EOD or (if intraday and first bar of session)
                    if (IsFirstTickOfBar && CurrentBar > 0 && (!Bars.BarsType.IsIntraday || (Bars.IsFirstBarOfSession && Bars.IsResetOnNewTradingDay)))
		            {
		                gap += (Open[0] - Close[1]);
		            }		        
                }
                // If not gapless, gap = 0 so that's okay.
                closeSeries[0] = Close[0] - gap;
                highSeries[0] = High[0] - gap;
                lowSeries[0] = Low[0] - gap;

		        if (CurrentBar == 0)
		            return;
                      
		        double high = highSeries[0];
		        double low = lowSeries[0];
                double close1 = closeSeries[1];
            
                atrSeries[0] = CalculateAtr(SqueezeLength, high, low, close1, atrSeries);

                double stdDev0 = stdDev[0];
		        double var0 = StdDevKeltner * atrSeries[0];
               
                if (var0.ApproxCompare(0) != 0)
                {
                    squeezeVar = StdDevBollinger * stdDev0 / var0;
                }
                if (squeezeVar < AlertLevel)
                {
                    // Squeeze occurs
                    Values[2][0] = 1d;
                    if (Values[2][1].ApproxCompare(0) == 0)
                        signalBarNum = CurrentBar;
                }
                else
                {
                    Values[2][0] = 0d;
                    if (Values[2][1].ApproxCompare(1) == 0)
                        signalBarNum = CurrentBar;
                }
                // Set bars ago. Include current bar
		        Values[3][0]= CurrentBar - signalBarNum + 1;

                switch (SqueezeMaType)
                {
                    case SqueezeMaTypeEnum.EMA:
                        maSeries[0] = ema[0];
                        break;
                    case SqueezeMaTypeEnum.DEMA:
                        maSeries[0] = dema[0];
                        break;
                    case SqueezeMaTypeEnum.TEMA:
                        maSeries[0] = tema[0];
                        break;
                }

                var linValue0 = closeSeries[0] - ((max[0] + min[0]) / 2.0 + maSeries[0]) * 0.5;

                linValueSeries[0] = linValue0;
                    
                var histogramValue0 = linReg[0] / tickSize;
               
                Value[0] = histogramValue0;

                var histogramValue1 = Value[1];

                if (histogramValue0 >= 0.0)
                {
                    if (histogramValue0 > histogramValue1)
                    {   //up
                        Values[1][0] = 2d;
                    }
                    else
                    {   //pullback
                        Values[1][0] = 1d;
                    }
                }
                else
                {
                    if (histogramValue0 < histogramValue1)
                    {   //down
                        Values[1][0] = -2d;
                    }
                    else
                    {   //bounce
                        Values[1][0] = -1d;
                    }
                }
                
            }
		    catch (Exception ex)
		    {
                Print(Name + ": " + ex);
		    }

		}

        #region Properties

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> HistogramValueSeries
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
		[XmlIgnore]
		public Series<double> HistogramDirectionSeries
		{
			get { return Values[1]; }
		}

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> SqueezeAlertSeries
        {
            get { return Values[2]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> NumberOfBarsSeries
        {
            get { return Values[3]; }
        }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Plot gapless?", GroupName = "01 Parameters.General", Order = 1)]
        public bool UseGapless { get; set; }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Use tick size?", Description = "Divide by 'tick size' to compare with other instruments", GroupName = "01 Parameters.General", Order = 2)]
        public bool UseTickSize { get; set; }

        [NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "MA type", GroupName = "02 Parameters.Histogram", Order = 1)]
        public SqueezeMaTypeEnum SqueezeMaType { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "MA length", GroupName = "02 Parameters.Histogram", Order = 2)]
        public int MaLength { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Linear Reg length", GroupName = "02 Parameters.Histogram", Order = 3)]
        public int LinRegLength { get; set; }

        [Range(1, double.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "BB value", GroupName = "03 Parameters.Squeeze", Order = 1)]
        public double StdDevBollinger { get; set; }

        [Range(1, double.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "K value", GroupName = "03 Parameters.Squeeze", Order = 2)]
        public double StdDevKeltner { get; set; }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Length", GroupName = "03 Parameters.Squeeze", Order = 3)]
        public int SqueezeLength { get; set; }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Sim22.Gapless.Sim22_SqueezeGaplessA[] cacheSim22_SqueezeGaplessA;
		public Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			return Sim22_SqueezeGaplessA(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength);
		}

		public Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(ISeries<double> input, bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			if (cacheSim22_SqueezeGaplessA != null)
				for (int idx = 0; idx < cacheSim22_SqueezeGaplessA.Length; idx++)
					if (cacheSim22_SqueezeGaplessA[idx] != null && cacheSim22_SqueezeGaplessA[idx].UseGapless == useGapless && cacheSim22_SqueezeGaplessA[idx].UseTickSize == useTickSize && cacheSim22_SqueezeGaplessA[idx].SqueezeMaType == squeezeMaType && cacheSim22_SqueezeGaplessA[idx].MaLength == maLength && cacheSim22_SqueezeGaplessA[idx].LinRegLength == linRegLength && cacheSim22_SqueezeGaplessA[idx].StdDevBollinger == stdDevBollinger && cacheSim22_SqueezeGaplessA[idx].StdDevKeltner == stdDevKeltner && cacheSim22_SqueezeGaplessA[idx].SqueezeLength == squeezeLength && cacheSim22_SqueezeGaplessA[idx].EqualsInput(input))
						return cacheSim22_SqueezeGaplessA[idx];
			return CacheIndicator<Sim22.Gapless.Sim22_SqueezeGaplessA>(new Sim22.Gapless.Sim22_SqueezeGaplessA(){ UseGapless = useGapless, UseTickSize = useTickSize, SqueezeMaType = squeezeMaType, MaLength = maLength, LinRegLength = linRegLength, StdDevBollinger = stdDevBollinger, StdDevKeltner = stdDevKeltner, SqueezeLength = squeezeLength }, input, ref cacheSim22_SqueezeGaplessA);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			return indicator.Sim22_SqueezeGaplessA(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength);
		}

		public Indicators.Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(ISeries<double> input , bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			return indicator.Sim22_SqueezeGaplessA(input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			return indicator.Sim22_SqueezeGaplessA(Input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength);
		}

		public Indicators.Sim22.Gapless.Sim22_SqueezeGaplessA Sim22_SqueezeGaplessA(ISeries<double> input , bool useGapless, bool useTickSize, SqueezeMaTypeEnum squeezeMaType, int maLength, int linRegLength, double stdDevBollinger, double stdDevKeltner, int squeezeLength)
		{
			return indicator.Sim22_SqueezeGaplessA(input, useGapless, useTickSize, squeezeMaType, maLength, linRegLength, stdDevBollinger, stdDevKeltner, squeezeLength);
		}
	}
}

#endregion
